/**
  *  \file client/dlg-build.h
  *
  *  (c) 2001-2008 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_DLG_BUILD_H
#define PCC_V2_CLIENT_DLG_BUILD_H

#include <vector>
#include "client/wcost.h"
#include "client/winfo.h"
#include "cpluslib/holder.h"
#include "game/action.h"
#include "game/spec.h"
#include "ui/listbox.h"
#include "ui/ui.h"
#include "ui/window.h"
#include "ui/widgets.h"

class GPlanet;

class WComponentListbox : public UIListbox {
 protected:
    GPlanet*   tick_planet;
 public:
    WComponentListbox(int id, int lines, int width, GPlanet* planet);

 protected:
    void drawComponent(GfxCanvas& can, int line, const GComponent& com, TTechLevel tech_area, int have);
};

class WHullListbox : public WComponentListbox {
    std::vector<int>  content;
 public:
    WHullListbox(int id, int lines, int width, GPlanet* planet);
    ~WHullListbox();

    void drawPart(GfxCanvas& can, int from, int to);

    int   getCurrentHull() const;
    void  addHullId(int n);
    void  swapContents(std::vector<int>& new_content);
    void  scrollToHull(int n);
};

class WEngineListbox : public WComponentListbox {
 public:
    WEngineListbox(int id, int width, GPlanet* tick_planet);
    void drawPart(GfxCanvas& can, int from, int to);
};

class WBeamListbox : public WComponentListbox {
 public:
    WBeamListbox(int id, int width, GPlanet* tick_planet);
    void drawPart(GfxCanvas& can, int from, int to);
};

class WLauncherListbox : public WComponentListbox {
 public:
    WLauncherListbox(int id, int width, GPlanet* tick_planet);
    void drawPart(GfxCanvas& can, int from, int to);
};

class WOrderDisplay : public UIWidget {
    TBuildOrder o;
 public:
    WOrderDisplay();
    void draw(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);
    void setBuildOrder(const TBuildOrder& o);
};

class WPartStorageDisplay : public UIWidget {
    int n;
 public:
    WPartStorageDisplay();
    void draw(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);
    void setCount(int n);
};

class WPartStorageGroup : public UIGroup {
    UIImageWidget image;
    WPartStorageDisplay psd;
    UISpacer      spacer;
    UIButton      button;
 public:
    WPartStorageGroup();
    ~WPartStorageGroup();

    void setCount(int n)
        { psd.setCount(n); }

    void setImage(Ptr<GfxPixmap> pix)
        { image.setImage(pix); }
};

class WPartCountInfo : public UIWidget {
    bool torps;
    int want, max;
 public:
    WPartCountInfo(bool torps);
    void draw(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);
    void setPartsMax(int n);
    void setPartsWanted(int n);
    int getPartsWanted() const
        { return want; }
    int getPartsMax() const
        { return max; }
};

class WPartCountGroup : public UIGroup {
    WPartCountInfo pci;
    UIGroup button_group;
    UIButton plus, minus;
 public:
    WPartCountGroup(bool torps);
    ~WPartCountGroup();

    bool handleCommand(const UICommand& cmd, UIWidget& sender);

    void setPartsMax(int n);
    void setPartsWanted(int n)
        { pci.setPartsWanted(n); }
    int getPartsWanted() const
        { return pci.getPartsWanted(); }
    int getPartsMax() const
        { return pci.getPartsMax(); }
};

class WEngineInfoDiagram : public UIWidget {
    int engine;
 public:
    WEngineInfoDiagram();
    void draw(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);
    void setEngine(int n);
};

class WEngineStorageInfo : public UIWidget {
    int have, need;
 public:
    WEngineStorageInfo();
    void setEnginesAvailable(int n);
    void setEnginesNeeded(int n);
    void draw(GfxCanvas& can);
    void getLayoutInfo(LayoutInfo& info);
};

class WShipBuildScreen : public UIWindow {
    WHullListbox      hull_list;
    WEngineListbox    engine_list;
    WBeamListbox      beam_list;
    WLauncherListbox  torp_list;

    WHullInfo         hull_info;
    WBeamInfo         beam_info;
    WTorpInfo         torp_info;
    WEngineInfo       engine_info;

    WPartStorageGroup hull_storage, torp_storage, beam_storage;
    WEngineStorageInfo engine_storage;
    WEngineInfoDiagram engine_diag;

    WPartCountGroup beam_count, torp_count;

 protected:
    GPlanetTransfer           transfer;
    GStarbaseBuildShipAction  action;

 private:
    WItemCostDisplay      item_cost_dpy;
    WOrderDisplay         order_dpy;

    UICardGroup       cards;
    UICardGroupTab    tabs;

    SignalConnection conn_update;

    void onPlanetChanged(int);

    // Must be declared exactly here so that it dies first.
 protected:
    ObjectHolder      h;

    void updateHullPage();
    void updateBeamPage();
    void updateEnginePage();
    void updateLauncherPage();
    void updateBuildOrder();
 public:
    WShipBuildScreen(string_t name, GPlanet& the_planet);
    ~WShipBuildScreen();

    bool handleCommand(const UICommand& cmd, UIWidget& sender);
};

class WShipBuildScreenWithActor : public WShipBuildScreen {
    void onOk();
    void onCancel();
    void onCancelBuild();
 public:
    WShipBuildScreenWithActor(GPlanet& the_planet);
    ~WShipBuildScreenWithActor();
};

#endif
