/**
  *  \file client/wcost.cc
  *
  *  (c) 2001-2008 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/wcost.h"
#include "game/cargo.h"
#include "ui/table.h"
#include "ui/uiglobal.h"
#include "util/nls.h"
#include "util/misc.h"

WItemCostDisplay::WItemCostDisplay(GCargoCostTransaction& action,
                                   bool include_item_in_cost)
    : action(action), cost(), have_tech(1), need_tech(1),
      include_item_in_cost(include_item_in_cost)
{
    setState(st_Disabled, false);
}

WItemCostDisplay::~WItemCostDisplay()
{ }

void
WItemCostDisplay::getLayoutInfo(LayoutInfo& info)
{
    // FIXME: should explicitly compute table size
    info.flags = 0;
    info.min_size = info.pref_size = GfxPoint(350, 7*font_heights[FONT_NORMAL]);
}

bool
WItemCostDisplay::handleEvent(const UIEvent& e, bool second_pass)
{
    return UIWidget::handleEvent(e, second_pass);
}

static void
writeCost(UIAbstractTableWriter& tw, int32 need, int32 remain)
{
    if (remain < 0)
        tw.writeVariableCell(UIColor::tc_Red, numToString(need));
    else
        tw.writeVariableCell(UIColor::tc_Green, numToString(need));
}

void
WItemCostDisplay::draw(GfxCanvas& can)
{
    UITableWriter tw(can, getSkin(), getExtent(), FONT_NORMAL);

    // FIXME: headers
    
    tw.newLColumn();
    tw.writeFixedCell(UIColor::tc_Static, string_t());
    tw.writeFixedCell(UIColor::tc_Static, _("Tritanium:"));
    tw.writeFixedCell(UIColor::tc_Static, _("Duranium:"));
    tw.writeFixedCell(UIColor::tc_Static, _("Molybdenum:"));
    tw.writeFixedCell(UIColor::tc_Static, _("Megacredits:"));
    tw.writeFixedCell(UIColor::tc_Static, _("Supplies:"));
    tw.writeFixedCell(UIColor::tc_Static, _("Tech Level:"));

    // "This part". Color is red if *total* resources don't suffice to
    // buy one of these.
    tw.newRColumn(5 * font_ems[FONT_NORMAL]);
    tw.writeFixedCell(UIColor::tc_Contrast, _("This Part"));
    writeCost(tw, cost.get(el_Tritanium),   action.getRemaining(el_Tritanium)  + action.getReserved(el_Tritanium) - cost.get(el_Tritanium));
    writeCost(tw, cost.get(el_Duranium),    action.getRemaining(el_Duranium)   + action.getReserved(el_Duranium) - cost.get(el_Duranium));
    writeCost(tw, cost.get(el_Molybdenum),  action.getRemaining(el_Molybdenum) + action.getReserved(el_Molybdenum) - cost.get(el_Molybdenum));
    writeCost(tw, cost.get(el_Money),       action.getRemaining(el_Money) + action.getReserved(el_Money) + action.getRemaining(el_Supplies) - cost.get(el_Money));
    if (cost.get(el_Supplies))
        writeCost(tw, cost.get(el_Supplies), action.getRemaining(el_Supplies) + action.getReserved(el_Supplies) - cost.get(el_Supplies));
    else
        tw.writeVariableCell(UIColor::tc_Green, string_t());
    writeCost(tw, need_tech, have_tech - need_tech);

    // "Total"
    // FIXME: If include_item_in_cost is set, we want to colorize an
    // item red if remaining stuff doesn't suffice to buy another one
    // of these.
    tw.newRColumn(5 * font_ems[FONT_NORMAL]);
    tw.writeFixedCell(UIColor::tc_Contrast, _("Total"));
    writeCost(tw, action.getCost(el_Tritanium), -action.getMissing(el_Tritanium));
    writeCost(tw, action.getCost(el_Duranium), -action.getMissing(el_Duranium));
    writeCost(tw, action.getCost(el_Molybdenum), -action.getMissing(el_Molybdenum));
    writeCost(tw, action.getCost(el_Money), -action.getMissing(el_Money));
    if (action.getCost(el_Supplies))
        writeCost(tw, action.getCost(el_Supplies), -action.getMissing(el_Supplies));
    else
        tw.writeVariableCell(UIColor::tc_Green, string_t());
    tw.writeVariableCell(UIColor::tc_Green, string_t());

    // Available
    tw.newRColumn(5 * font_ems[FONT_NORMAL]);
    tw.writeFixedCell(UIColor::tc_Contrast, _("Available"));
    writeCost(tw, action.getReserved(el_Tritanium)  + action.getRemaining(el_Tritanium), 0);
    writeCost(tw, action.getReserved(el_Duranium)   + action.getRemaining(el_Duranium), 0);
    writeCost(tw, action.getReserved(el_Molybdenum) + action.getRemaining(el_Molybdenum), 0);
    writeCost(tw, action.getReserved(el_Money)      + action.getRemaining(el_Money), 0);
    writeCost(tw, action.getReserved(el_Supplies)   + action.getRemaining(el_Supplies), 0);
    writeCost(tw, have_tech, 0);

    tw.newLColumn();
    tw.writeFixedCell(UIColor::tc_Green, string_t());
    tw.writeFixedCell(UIColor::tc_Green, _("kt"));
    tw.writeFixedCell(UIColor::tc_Green, _("kt"));
    tw.writeFixedCell(UIColor::tc_Green, _("kt"));
    tw.writeFixedCell(UIColor::tc_Green, _("mc"));
    tw.writeFixedCell(UIColor::tc_Green, _("kt"));

    tw.finish();

    can.update(getExtent());
}

void
WItemCostDisplay::setCost(const GCost& it)
{
    cost = it;
    drawWidget(false);
}

void
WItemCostDisplay::setTech(int have, int need)
{
    have_tech = have;
    need_tech = need;
    drawWidget(false);
}
