/**
  *  \file game/cost.h
  *  \brief Cost/Resource Set
  *
  *  (c) 2001-2008 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_COST_H
#define PCC_V2_GAME_COST_H

#include "game/struct.h"
#include "game/cargotype.h"
#include "cpluslib/string.h"

/** Resource amount/cost. Represents the resource cost of a unit
    (weapon, building), or a corresponding resource set, and provides
    operations for manipulating it.

    This amount can be <em>valid</em> or <em>invalid</em>. It becomes
    invalid if a member is negative, that is, if a large value (the
    price) is subtracted from a small one (available resource). */
class GCost {
    int32 t, d, m, mc, sup;
 public:
    GCost();
    GCost(const TCost& cost);
    GCost& operator+=(const GCost& other);
    GCost& operator-=(const GCost& other);
    GCost& operator*=(int n);
    GCost  operator*(int n) const;
    bool   operator==(const GCost& other) const;
    bool   operator!=(const GCost& other) const
        { return !operator==(other); }

    void set(GCargoType type, int32 n);
    int32 get(GCargoType type) const;
    
    bool isEnoughFor(const GCost& other) const;
    bool isValid() const;

    static GCost fromString(string_t value);
};

#endif
