/**
  *  \file game/expl.h
  *  \brief Explosions
  *
  *  (c) 2001-2008 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_EXPL_H
#define PCC_V2_GAME_EXPL_H

#include <list>
#include "game/coord.h"
#include "cpluslib/string.h"

/** An explosion. Maintains position and optional additional information
    attached with an explosion. */
class GExplosion {
    int id;                     ///< Id of explosion, if known.
    GPoint pos;                 ///< Position of explosion.
    string_t ship_name;         ///< Name of exploded ship, if known.
    int ship_id;                ///< Id of exploded ship, if known.
 public:
    GExplosion(int id, GPoint pos);
    ~GExplosion();

    GPoint getPos() const;
    string_t getShipName() const;
    int getShipId() const;
    int getId() const;

    void setPos(GPoint pos);
    void setShipName(string_t name);
    void setShipId(int id);

    bool merge(const GExplosion& other);
};

/** Explosion container. Maintains a set of explosions. */
class GExplosionContainer {
    typedef std::list<GExplosion> container_t;
    container_t explosions;

 public:
    typedef container_t::iterator iterator;
    typedef container_t::const_iterator const_iterator;

    GExplosionContainer();
    ~GExplosionContainer();

    GExplosion& addExplosion(GExplosion x);

    iterator begin()
        { return explosions.begin(); }
    const_iterator begin() const
        { return explosions.begin(); }
    iterator end()
        { return explosions.end(); }
    const_iterator end() const
        { return explosions.end(); }
};

#endif
