/*
 *  TEST: expression evaluation
 *
 *  This takes a number of expressions on the command line.
 *  Each of them is evaluated; the result and byte-compilation result
 *  is displayed.
 */

#include <iostream>
#include "int/lex.h"
#include "int/data.h"
#include "int/expr.h"
#include "util/debug.h"
#include "int/bytecode.h"
#include "int/exec.h"

class TestContext : public IntContext {
    static IntObject values[26];
 public:
    handle_t  lookup(string_t name);
    handle_t  lookupSub(string_t name) { return -1; }
    void      call(handle_t, int, IntExecutionContext&) { }
    IntObject read(handle_t);
    void      store(handle_t, IntObject);
    TestContext* clone() { return new TestContext(); }
    bool next() { return false; }
};

IntObject TestContext::values[26];

TestContext::handle_t
TestContext::lookup(string_t name)
{
    if (name.length() == 1 && name[0] >= 'A' && name[0] <= 'Z')
        return name[0] - 'A';
    else
        return -1;
}

IntObject
TestContext::read(handle_t n)
{
    return values[n];
}

void
TestContext::store(handle_t n, IntObject v)
{
    values[n] = v;
}

static void
show(IntObject o) {
    switch(o.getType()) {
     case res_Null:
        std::cout << "Empty\n";
        break;
     case res_Int:
        std::cout << "Int: " << o.getIntVal() << "\n";
        break;
     case res_Bool:
        std::cout << "Bool: " << o.getIntVal() << "\n";
        break;
     case res_Float:
        std::cout << "Float: " << o.getFloatVal() << "\n";
        break;
     case res_String:
        std::cout << "String: " << o.getStringVal() << "\n";
        break;
     case res_Context:
        std::cout << "Context\n";
        break;
     case res_Function:
        std::cout << "Function\n";
        break;
     default:
        std::cout << "UFO (type " << int(o.getType()) << ")\n";
        break;
    }
}

int main(int argc, char** argv)
{
    initDebug();
    while (*++argv) {
        try {
            IntExecutionContext con;
            con.pushContext(new TestContext());
            IntLexer lex(*argv);
            lex.lex();
            IntExprParser parser(lex);
            parser.parseExpr();
            IntExprNodePtr node = parser.pop();
//             try {
//                 IntObject o = node->eval(con);
//                 std::cout << *argv << "\n-> ";
//                 show(o);
//             }
//             catch(IntException& e) {
//                 std::cout << *argv << "\n-> Eval error: " << e.what() << "\n";
//             }
            {
                std::cout << "=== Sideeffect Code ===\n";
                IntByteCodeObject& bco = IntByteCodeObject::allocBCO();
                node->compileS(bco);
                bco.dump(std::cout);
            }
            {
                std::cout << "=== Value Code ===\n";
                IntByteCodeObject& bco = IntByteCodeObject::allocBCO();
                node->compileV(bco);
                bco.dump(std::cout);
                std::cout << "Running...";
                con.pushFrame(new IntFrame(bco, 0));
                try {
                    con.run();
                    std::cout << " -> ";
                    show(con.popV());
                }
                catch(IntException& e) {
                    std::cout << " -> Run error: " << e.what() << "\n";
                }
            }
        }
        catch(IntException& e) {
            std::cout << *argv << "\n-> Error: " << e.what() << "\n";
        }
    }
}
