#include "game/cmds.h"
#include "u/t_game.h"

#include <iostream>

static void
testOneCommand(const char* cmd, GCommand::Type type, int id, const char* arg, const char* result = 0,
               bool file_only = false)
{
    if (!arg)
        arg = cmd;
    try {
        TS_ASSERT(!GCommand::isMessageIntroducer(cmd));

        GCommand* p = GCommand::parseCommand(cmd, true);
        if (type == GCommand::phc_Other) {
            TS_ASSERT(!p);
        } else {
            TS_ASSERT(p);
            TS_ASSERT_EQUALS(p->getCommand(), type);
            TS_ASSERT_EQUALS(p->getId(), id);
            TS_ASSERT_EQUALS(p->getArg(), arg);
            if (result)
                TS_ASSERT_EQUALS(p->getCommandText(), result);
            delete p;
        }

        p = GCommand::parseCommand(cmd, false);
        if (file_only || type == GCommand::phc_Other) {
            TS_ASSERT(!p);
        } else {
            TS_ASSERT(p);
            TS_ASSERT_EQUALS(p->getCommand(), type);
            TS_ASSERT_EQUALS(p->getId(), id);
            TS_ASSERT_EQUALS(p->getArg(), arg);
            if (result)
                TS_ASSERT_EQUALS(p->getCommandText(), result);
            delete p;
        }
    }
    catch (...) {
        TS_FAIL("exception!");
    }
}

void
GameCommandTestSuite::testCommands()
{
    testOneCommand("language klingon",   GCommand::phc_Language, 0, "klingon", "language klingon");
    testOneCommand("lanGuaGE   klingon", GCommand::phc_Language, 0, "klingon", "language klingon");
    testOneCommand("l k",                GCommand::phc_Language, 0, "k", "language k");
    testOneCommand("la k",               GCommand::phc_Language, 0, "k", "language k");
    testOneCommand("li k",               GCommand::phc_Other, 0, 0);
    testOneCommand("phost: la k",        GCommand::phc_Language, 0, "k", "language k");

    testOneCommand("send config",        GCommand::phc_SendConfig, 0, "", "send config");
    testOneCommand("s c",                GCommand::phc_SendConfig, 0, "", "send config");
    testOneCommand("se CO",              GCommand::phc_SendConfig, 0, "", "send config");
    testOneCommand("send fcodes",        GCommand::phc_SendFcodes, 0, "", "send fcodes");
    testOneCommand("send f",             GCommand::phc_SendFcodes, 0, "", "send fcodes");
    testOneCommand("send racenames",     GCommand::phc_SendRacenames, 0, "", "send racenames");
    testOneCommand("s r",                GCommand::phc_SendRacenames, 0, "", "send racenames");
    testOneCommand("send money",         GCommand::phc_Other, 0, 0);

    testOneCommand("racename long Klingons", GCommand::phc_SetRaceName, GCommand::phcrn_Long, "Klingons", "race long Klingons");
    testOneCommand("ra       long    Klingons", GCommand::phc_SetRaceName, GCommand::phcrn_Long, "Klingons", "race long Klingons");
    testOneCommand("ra short Frogs",     GCommand::phc_SetRaceName, GCommand::phcrn_Short, "Frogs", "race short Frogs");
    testOneCommand("ra s Frogs",         GCommand::phc_SetRaceName, GCommand::phcrn_Short, "Frogs", "race short Frogs");
    testOneCommand("ra a Frogs",         GCommand::phc_SetRaceName, GCommand::phcrn_Adjective, "Frogs", "race adj Frogs");
    testOneCommand("ra adjective Frogs", GCommand::phc_SetRaceName, GCommand::phcrn_Adjective, "Frogs", "race adj Frogs");
    testOneCommand("r adj Foo",          GCommand::phc_Other, 0, 0);

    testOneCommand("filter y",           GCommand::phc_Filter, 0, "y", "filter y");
    testOneCommand("f y",                GCommand::phc_Filter, 0, "y", "filter y");

    testOneCommand("allies config 7 +c", GCommand::phc_ConfigAlly, 7, "+c", "allies config 7 +c");
    testOneCommand("a c 7 +c",           GCommand::phc_ConfigAlly, 7, "+c", "allies config 7 +c");

    testOneCommand("allies add 7",       GCommand::phc_AddDropAlly, 7, "add", "allies add 7");
    testOneCommand("a a 7",              GCommand::phc_AddDropAlly, 7, "a", "allies a 7");
    testOneCommand("a frob 7",           GCommand::phc_Other, 0, 0);

    testOneCommand("give ship 9 to 2",   GCommand::phc_GiveShip, 9, "2", "give ship 9 to 2");
    testOneCommand("g s 9 2",            GCommand::phc_GiveShip, 9, "2", "give ship 9 to 2");
    testOneCommand("give planet 9 2",    GCommand::phc_GivePlanet, 9, "2", "give planet 9 to 2");
    testOneCommand("g p 9 to 2",         GCommand::phc_GivePlanet, 9, "2", "give planet 9 to 2");
    testOneCommand("give foo 9 to 2",    GCommand::phc_Other, 0, 0);
    testOneCommand("give planet 9",      GCommand::phc_Other, 0, 0);

    testOneCommand("remote control 333", GCommand::phc_RemoteControl, 333, "control", "remote control 333");
    testOneCommand("re c 333",           GCommand::phc_RemoteControl, 333, "c", "remote c 333");
    testOneCommand("remote allow   333", GCommand::phc_RemoteControl, 333, "allow", "remote allow 333");
    testOneCommand("re a 333",           GCommand::phc_RemoteControl, 333, "a", "remote a 333");
    testOneCommand("remote forbid 333",  GCommand::phc_RemoteControl, 333, "forbid", "remote forbid 333");
    testOneCommand("re f 333",           GCommand::phc_RemoteControl, 333, "f", "remote f 333");
    testOneCommand("remote drop 333",    GCommand::phc_RemoteControl, 333, "drop", "remote drop 333");
    testOneCommand("re d 333",           GCommand::phc_RemoteControl, 333, "d", "remote d 333");
    testOneCommand("re blurb 333",       GCommand::phc_Other, 0, 0);

    testOneCommand("remote allow default", GCommand::phc_RemoteDefault, 0, "allow", "remote allow default");
    testOneCommand("re a d",             GCommand::phc_RemoteDefault, 0, "a", "remote a default");
    testOneCommand("remote forbid default", GCommand::phc_RemoteDefault, 0, "forbid", "remote forbid default");
    // testOneCommand("remote drop default", GCommand::phc_Other, 0, 0);

    testOneCommand("beamup 999 c100 nmax", GCommand::phc_Beamup, 999, "c100 nmax", "beamup 999 c100 nmax");
    testOneCommand("be  999 c100 nmax",  GCommand::phc_Beamup, 999, "c100 nmax", "beamup 999 c100 nmax");
    testOneCommand("beamup 140 T999 D999 M999 S999 C999 $999 N99",  GCommand::phc_Beamup, 140, "T999 D999 M999 S999 C999 $999 N99", "be 140 T999 D999 M999 S999 C999 $999 N99");
    testOneCommand("beamup 140 T999 D99 M99 S999 C999 $999 N99",  GCommand::phc_Beamup, 140, "T999 D99 M99 S999 C999 $999 N99", "beam 140 T999 D99 M99 S999 C999 $999 N99");
    testOneCommand("b 999 c100 nmax",    GCommand::phc_Other, 0, 0);

    testOneCommand("$thost-allies ee9ffa", GCommand::phc_TAlliance, 0, "ee9ffa", "$thost-allies ee9ffa", true);

    testOneCommand("$send-file foo",     GCommand::phc_SendFile, 0, "foo", "$send-file foo", true);
    testOneCommand("$send-f foo",        GCommand::phc_SendFile, 0, "foo", "$send-file foo", true);
    testOneCommand("$send- foo",         GCommand::phc_Other, 0, 0);

    testOneCommand("enemies add 8",      GCommand::phc_Enemies, 8, "add", "enemies add 8");
    testOneCommand("e a 8",              GCommand::phc_Other, 0, 0);    /* interpreted as bogus 'extmission' */
    testOneCommand("ene a 8",            GCommand::phc_Enemies, 8, "a", "enemies a 8");
    testOneCommand("ene d 8",            GCommand::phc_Enemies, 8, "d", "enemies d 8");
    testOneCommand("ene x 8",            GCommand::phc_Other, 0, 0);    /* not a valid 'enemies' subcommand */

    testOneCommand("unload 42 $10",      GCommand::phc_Unload, 42, "$10", "unload 42 $10");
    testOneCommand("unload 140 T999 D999 M999 S999 C999 $999 N99",  GCommand::phc_Unload, 140, "T999 D999 M999 S999 C999 $999 N99", "unl 140 T999 D999 M999 S999 C999 $999 N99");

    testOneCommand("transfer 42 $10",    GCommand::phc_Transfer, 42, "$10", "transfer 42 $10");
    testOneCommand("transfer 140 T999 D999 M999 S999 C999 $999 N99",  GCommand::phc_Transfer, 140, "T999 D999 M999 S999 C999 $999 N99", "tra 140 T999 D999 M999 S999 C999 $999 N99");
}

void
GameCommandTestSuite::testMI()
{
    TS_ASSERT(GCommand::isMessageIntroducer("message 1 2 3"));
    TS_ASSERT(GCommand::isMessageIntroducer("message 1"));
    TS_ASSERT(GCommand::isMessageIntroducer("message u"));
    TS_ASSERT(GCommand::isMessageIntroducer("m u"));
    TS_ASSERT(GCommand::isMessageIntroducer("rumor u"));
    TS_ASSERT(GCommand::isMessageIntroducer("rumour u"));
    TS_ASSERT(GCommand::isMessageIntroducer("ru u"));
    TS_ASSERT(!GCommand::isMessageIntroducer("r u"));
}

void
GameCommandTestSuite::testContainer()
{
    GCommand* null_command = 0;
    GCommandContainer c;
    TS_ASSERT_EQUALS(c.begin(), c.end());
    TS_ASSERT_DIFFERS(c.setCommand(GCommand::phc_ConfigAlly, 9, "+c"), null_command);
    TS_ASSERT_DIFFERS(c.setCommand(GCommand::parseCommand("a a 9", true)), null_command);
    TS_ASSERT_DIFFERS(c.begin(), c.end());

    GCommandContainer::iterator i = c.begin();
    TS_ASSERT_EQUALS((*i)->getCommand(), GCommand::phc_AddDropAlly);
    TS_ASSERT_EQUALS(c.getCommand(GCommand::phc_AddDropAlly, 9), *i);
    TS_ASSERT_EQUALS(c.getCommand(GCommand::phc_AddDropAlly, 7), null_command);
    TS_ASSERT_EQUALS(c.getCommand(GCommand::phc_AddDropAlly, 0), null_command);

    ++i;
    TS_ASSERT_DIFFERS(i, c.end());
    TS_ASSERT_EQUALS((*i)->getCommand(), GCommand::phc_ConfigAlly);
    TS_ASSERT_EQUALS(c.getCommand(GCommand::phc_ConfigAlly, 9), *i);

    ++i;
    TS_ASSERT_EQUALS(i, c.end());

    // remove 'allies config 9', test again
    TS_ASSERT(c.removeCommand(GCommand::phc_ConfigAlly, 9));
    TS_ASSERT_EQUALS(c.getCommand(GCommand::phc_ConfigAlly, 9), null_command);
    i = c.begin();
    TS_ASSERT_DIFFERS(i, c.end());
    TS_ASSERT_EQUALS((*i)->getCommand(), GCommand::phc_AddDropAlly);
    ++i;
    TS_ASSERT_EQUALS(i, c.end());

    // add 'allies config 9 +c', test again
    TS_ASSERT(c.setCommand(GCommand::phc_ConfigAlly, 9, "+c"));
    i = c.begin();
    TS_ASSERT_DIFFERS(i, c.end());
    TS_ASSERT_EQUALS((*i)->getCommand(), GCommand::phc_AddDropAlly);
    ++i;
    TS_ASSERT_DIFFERS(i, c.end());
    TS_ASSERT_EQUALS((*i)->getCommand(), GCommand::phc_ConfigAlly);
    ++i;
    TS_ASSERT_EQUALS(i, c.end());

    // test inquiry / replacement
    const GCommand* cmd = c.setCommand(GCommand::phc_Language, 0, "en");
    TS_ASSERT(cmd);
    TS_ASSERT_EQUALS(c.getCommand(GCommand::phc_Language, 0), cmd);
    cmd = c.setCommand(GCommand::phc_Language, 0, "de");
    TS_ASSERT(cmd);
    TS_ASSERT_EQUALS(c.getCommand(GCommand::phc_Language, 0), cmd);
    for (i = c.begin(); i != c.end(); ++i)
        if ((*i)->getCommand() == GCommand::phc_Language)
            TS_ASSERT_EQUALS((*i)->getArg(), "de");

    c.clear();
    TS_ASSERT_EQUALS(c.getCommand(GCommand::phc_AddDropAlly, 9), null_command);
    TS_ASSERT_EQUALS(c.begin(), c.end());
}
